use actix_files::Files;
use actix_web::{get, http, post, web, App, HttpResponse, HttpServer, Responder};
use sqlite::{Connection, State, Error};
use serde::{Serialize, Deserialize};

#[actix_web::main]
async fn main() -> Result<(), std::io::Error> {
    let conn = Connection::open("./data/data.db");

    HttpServer::new(|| {
        App::new()
            .service(hello)
            .service(get_articles)
            .service(Files::new("/", "./public").index_file("index.html"))
    })
    .bind(("0.0.0.0", 8080))?
    .run()
    .await
}


#[derive(serde::Serialize)]
struct Article {
    id: i64,
    title: String,
    auteur: Option<String>,
    edited_at: Option<String>,
    published_at: Option<String>,
    content: String,
}

#[get("/api/hello")]
async fn hello() -> impl Responder {
    HttpResponse::Ok().body("Hello world!")
}

#[get("/api/articles")]
async fn get_articles() -> impl Responder {
    let conn = Connection::open("./data/data.db").unwrap();
    let mut stmt = conn.prepare("SELECT * FROM articles").unwrap();
    let mut articles = Vec::new();

    while let State::Row = stmt.next().unwrap() {
        let id = stmt.read::<i64, _>(0).unwrap();
        let title = stmt.read::<String, _>(1).unwrap();
        let content = stmt.read::<String, _>(3).unwrap();
        articles.push(Article {
            id,
            title,
            auteur: None,
            edited_at: None,
            published_at: None,
            content
        });
    }

    HttpResponse::Ok().json(articles)
}
